import { Api, Status } from './types'
const { types } = require('@joystream/types')

const { Block } = require('./db/models')

const { ApiPromise, WsProvider } = require('@polkadot/api')
// TODO allow alternative backends
const wsLocation = 'ws://localhost:9944'
// 'wss://rome-rpc-endpoint.joystream.org:9944'

const { addBlock } = require('./joystream')

module.exports = (io) => {
  const handleUpstream = async (api: Api) => {
    let status: Status = {}
    api.derive.chain.subscribeNewHeads(
      async (header) => (status = await addBlock(api, io, header, status))
    )
  }

  io.on('connection', async (socket) => {
    console.log(chalk.green(`[socket.io] Connection: ${socket.id}`))

    socket.on('get blocks', async (limit) => {
      const blocks = await Block.findAll({ limit, order: [['id', 'DESC']] })
      socket.emit('blocks', blocks)
    })

    socket.on('disconnect', async () => {
      console.log(chalk.red(`[socket.io] Disconnect: ${socket.id}`))
    })
  })

  const connectUpstream = async () => {
    try {
      console.debug(`[Joystream] Connecting to ${wsLocation}`)
      const provider = new WsProvider(wsLocation)
      const api = await ApiPromise.create({ provider, types })
      await api.isReady
      console.debug(`[Joystream] Connected.`)
      handleUpstream(api)
    } catch (e) {
      //console.log(`[Joystream] upstream connection failed`, e)
    }
  }
  connectUpstream()
}
