import React from 'react'
import { Layout, Loading, Main, Modals, Player } from './components'
import { withRouter } from 'react-router-dom'
import socket from './socket'

const initialState = {
  loading: false,
  blocks: [],
  timestamp: 0,
  modal: null,
  validators: [],
  validator: null,
}

class App extends React.Component {
  initializeSocket() {
    socket.on('connect', () => {
      if (!socket.id) return console.log('no websocket connection')
      console.log('my socketId:', socket.id)
      socket.emit('get blocks', 24 * 600)
    })
    socket.on('blocks', (newblocks) => {
      const blocks = this.state.blocks
        .filter((b) => !newblocks.find((blk) => blk.id === b.id))
        .concat(newblocks)
        .sort((a, b) => b.id - a.id)
      this.setState({ loading: false, blocks }) // TODO
    })
    socket.on('block', (block) => {
      const blocks = this.state.blocks
        .filter((b) => b.id !== block.id)
        .concat(block)
        .sort((a, b) => b.id - a.id)
      this.setState({ loading: false, blocks, timestamp: block.timestamp })
    })
  }

  setShowModal(showModal, vars = {}) {
    this.setState({ showModal, ...vars })
  }
  showValidator(account) {
    const { validators } = this.state
    const v = { account }
    const validator = validators.find((v) => v.account === account) || v
    this.setShowModal('validator', { validator })
  }

  renderError() {
    if (this.state.showModal === 'Error') return
    this.setShowModal('Error')
  }
  renderApp() {
    return (
      <div className="w-100 h-100 d-flex flex-column overflow-hidden bg-dark">
        <Modals
          setShowModal={this.setShowModal}
          showModal={this.state.showModal}
          validator={this.state.validator}
          blocks={this.state.blocks}
        />
        <Main
          setShowModal={this.setShowModal}
          showValidator={this.showValidator}
          {...this.state}
        />
      </div>
    )
  }
  render() {
    if (this.state.loading) return <Loading />
    if (this.state.error) this.renderError()
    if (this.state.component === 'layout') return <Layout {...this.state} />
    //return <Player />
    return this.renderApp()
  }

  componentDidMount() {
    this.initializeSocket()
    //setInterval(() => this.setState({}), 100)
  }
  componentWillUnmount() {
    console.log('unmounting...')
  }
  constructor() {
    super()
    this.state = initialState
    this.setShowModal = this.setShowModal.bind(this)
    this.showValidator = this.showValidator.bind(this)
  }
}

export default withRouter(App)
