import React from 'react'
import { Badge, Button, Modal } from 'react-bootstrap'
import moment from 'moment'

const Validator = (props) => {
  const { account, handle = '' } = props.validator

  const blocks = props.blocks.filter((b) => b.author === account)
  const title = handle === '' ? account : handle
  const blocktime =
    blocks.map((b) => b.blocktime).reduce((a, b) => a + b, 0) / blocks.length
  const time = blocks[blocks.length - 1].timestamp
  const firstBlock = moment(time).fromNow()

  return (
    <Modal
      size="lg"
      aria-labelledby="contained-modal-title-vcenter"
      centered
      show={props.show}
      onHide={props.onHide}
    >
      <Modal.Header className="no-border" closeButton>
        <Modal.Title id="contained-modal-title-vcenter ">{title}</Modal.Title>
      </Modal.Header>
      <Modal.Body className="text-center">
        <div className="p-3 font-weight-bold">
          <div>
            Blocks: {blocks.length} (since {firstBlock})
          </div>
          <div>Average Blocktime: {(blocktime / 1000).toFixed(3)} s</div>
          {blocks.map((b) => (
            <Badge variant={b.blocktime > 7000 ? 'warning' : 'light'}>
              {b.id}: {b.blocktime}
            </Badge>
          ))}
        </div>
      </Modal.Body>
      <Modal.Footer>
        <Button variant="dark" onClick={props.onHide}>
          Close
        </Button>
      </Modal.Footer>
    </Modal>
  )
}

export default Validator
